/*********************************************************************************
CCD Output - braille project C5402 target software
Andrew Paquale 09913770
26/7/2002
**********************************************************************************/

#include <type.h>
#include <board.h>
#include <codec.h>
#include <mcbsp54.h>
#include <stdio.h>
#include "target.h"
#include "intr.h"

void codewheel_interrupt0();
void SI_interrupt2();

/*****************************************************************************/
/* Global Variables                                                          */
/*****************************************************************************/
HANDLE hHandset;
s16 data;
int data_array[60];
static short codewheel_var = 0;
int i = 0, newline_var = 0;
int reference_amplitude;		
int sam32dot1, sam33dot1, sam34dot1, sam35dot1, sam36dot1;	//sample number, fuzzy dot variable
int sam38dot2, sam39dot2, sam40dot2, sam41dot2, sam42dot2;	//sample number, fuzzy dot variable
int sam44dot3, sam45dot3, sam46dot3, sam47dot3, sam48dot3;	//sample number, fuzzy dot variable
int dot1_present, dot2_present, dot3_present;
char temp[80];

extern int bio_test()
{
asm("_test_bio:");
asm("	LD #0, A ;Set return value to false ");
asm("	BC bio_low, NBIO ;NBIO = high");
asm("	;BIO = 0 => moving forward");
asm("	NOP");
asm("	NOP ");
asm("	LD #1, A ;Set return value to true ");
asm("	RET ;Card present, return normally");
asm("bio_low: ;bio is low ");
asm("	RET ;exit");
}

void setup_interrupt()
{
	asm(" SSBX INTM");	// Globally disable interrupts
	asm(" STM #5,1h");	// Clear any previous interrupt in IFR
	asm(" STM #4,0h");	// Enable the int2 interrupt, disable codewheel(int0) in IMR
	asm(" RSBX INTM");	// Globally enable interrupts
}
/*****************************************************************************************
--------------------------------------- main function ------------------------------------
1. set up interrupts
2. set up tlc320AD50 (initailise MsCbs serial port, set sampling rate etc)
3. wait for events
*****************************************************************************************/

void main()
{
    
    if (brd_init(100))
        return;

//-------------------------1. set up interrupts-------------------------------------------	
	
	INTR_INIT								//initailise pointer to vector table
	INTR_HOOK( 16,codewheel_interrupt0)		//put ISR into vector table
	INTR_HOOK( 18,SI_interrupt2)			//put ISR into vector table
	
	// Setup the interrupt handler
	setup_interrupt();
	port0 = 0x80;			//sets CNTL1 register, enable int2
	
//-------------------------2. Setup the ADC ---------------------------------------------	
	
    /* Open Handset Codec */
    hHandset = codec_open(HANDSET_CODEC);               // Acquire handle to codec 

    // Set codec parameters 
    codec_adc_mode(hHandset, CODEC_ADC_15BIT);          // ADC in 15-bit mod
    codec_ain_gain(hHandset, CODEC_AIN_0dB);            // 0dB gain on analog input to ADC 
    codec_sample_rate(hHandset,SR_16000);               // 16KHz sampling rate 


while (1);
    /* Polling and digital loopback */
    
	return;
}


/**************************************************************************************
---------------------- codewheel_interrupt --------------------------------------------
The SI Pulse provides an interrupt to X_INT1 every 5.3 mS, 
the DSP will only process the CCD image slice from the AIC if 
a X_INT0 event (indicating a codewheel trigger signal) 
occurred in the last 5.3 mS AND XF equals zero (indicating forward motion)
***************************************************************************************/
interrupt void codewheel_interrupt0()
{
asm(" SSBX INTM");		// Disable interrupts


codewheel_var = bio_test();		//test to see if codewheel is moving forward
if (codewheel_var == 1) { 		//enable the SI interrupt
	brd_led_enable(BRD_LED0); 
	brd_led_enable(BRD_LED1);
	asm("	SSBX 1, 13");
	asm("	STM #1h,IFR");	// Clear the interrupt0 so may be used again (IFR)
	asm(" 	STM #4h,0h");		//but disable it until SI interrupt service routine 
	}							//completes,and enable SI interrupt 1 (IMR)
	
if (codewheel_var == 0)	{ 		//keep waiting, dont enable SI interrupt
	brd_led_disable(BRD_LED0); 
	brd_led_disable(BRD_LED1);
	brd_led_disable(BRD_LED2);
	}
asm(" STM #5,1h");	// Clear any previous interrupt in IFR
asm(" STM #4,0h");	// Enable the int2 interrupt, disable codewheel(int0) in IMR
asm(" RSBX INTM");		//globally enable interrupts

}

/************************************************************************************
-------------------- SI_interrupt ---------------------------------------------------
1. record the CCD output (57 samples after SI pulse)
2. determine dots within image slice of 57 samples
3. store then correlate with previous or latter slices constituting a braille cell
4. determine braille cell
5. send to host(use rtdx, does not require dsp/bios)
*************************************************************************************/
interrupt void SI_interrupt2()
{

asm(" SSBX INTM");		// Disable interrupts

brd_led_enable(BRD_LED2);
//----------------------- 1. record the CCD output (57 samples after SI pulse)--------------	
while (i < 59)
    {
       /* Wait for sample from handset */
       while (!MCBSP_RRDY(HANDSET_CODEC)) {};

       /* Read sample from and write back to handset codec */
       data = *(volatile u16*)DRR1_ADDR(HANDSET_CODEC);
       data_array[i] = data;
	   ++i;
    }

/*--------------------2. determine dots within image slice of 57 samples--------------------
  a. use at least 3 image slices to detemine dots in a "slice', since ccd
 	 may not be perfectly horizontal
  b. apply fuzzy logic fundamentals to determine dots within sample array*/
 i = 0;	// first 20 samples are out of lens view
 while (i < 58) {
 	if (newline_var == 20) {printf("\n"); newline_var = 0;}	//prevent output window horizontal overflow
	printf(" %d",  data_array[i]);
	++i;
	++newline_var;
	}
	
reference_amplitude = (data_array[28] + data_array[29] + data_array[30]) / 3;
//to determine if a dot 1 exists (by shadow)
sam32dot1 = (reference_amplitude - data_array[32]) /5; 	//5 is fuzzy variable
sam33dot1 = (reference_amplitude - data_array[33]) /2; 	
sam34dot1 = (reference_amplitude - data_array[34]) /1; 	
sam35dot1 = (reference_amplitude - data_array[35]) /2; 	
sam36dot1 = (reference_amplitude - data_array[36]) /5; 		
dot1_present = (sam32dot1 + sam33dot1 + sam34dot1 + sam35dot1 + sam36dot1) / 5;		//divide by 5 to prevent overflow

//to determine if a dot 2 exists (by shadow)
sam38dot2 = (reference_amplitude - data_array[38]) /5; 	//5 is fuzzy variable
sam39dot2 = (reference_amplitude - data_array[39]) /2; 	
sam40dot2 = (reference_amplitude - data_array[40]) /1; 	
sam41dot2 = (reference_amplitude - data_array[41]) /2; 	
sam42dot2 = (reference_amplitude - data_array[42]) /5; 		
dot2_present = (sam38dot2 + sam39dot2 + sam40dot2 + sam41dot2 + sam42dot2) / 5;		//divide by 5 to prevent overflow

//to determine if a dot 3 exists (by shadow)
sam44dot3 = (reference_amplitude - data_array[44]) /5; 	//5 is fuzzy variable
sam45dot3 = (reference_amplitude - data_array[45]) /2; 	
sam46dot3 = (reference_amplitude - data_array[46]) /1; 	
sam47dot3 = (reference_amplitude - data_array[47]) /2; 	
sam48dot3 = (reference_amplitude - data_array[48]) /5; 		
dot3_present = (sam44dot3 + sam45dot3 + sam46dot3 + sam47dot3 + sam48dot3) / 5;		//divide by 5 to prevent overflow

sprintf(temp, "\n sprintf is working %d %d %d", dot1_present, dot2_present, dot3_present);
puts (temp);
brd_led_toggle(BRD_LED1);

/*-----3. store then correlate with previous or latter slices constituting a braille cell---
	- Since initially the ccd could be placed anywhere on the braille line, the first image 
	slice will only be used if the next image slice contituting dots is inside the width
	of a braille cell (determined by counting the codewheel interrupts) otherwise it is
	discarded (as being the the second row of a braille cell)
	*/
//-------------------------------4. determine braille cell---------------------------------
/*-----------------------------5. send to host(use rtdx, does not require dsp/bios)-------*/
asm(" STM #5,1h");	// Clear any previous interrupt in IFR
asm(" STM #1,0h");		// Enable the int0 interrupt, disable SI(int1) in IMR
asm(" SSBX INTM");		// globally disable interrupts		
}
